// Use Kit
import { extensor, foreach, hand, cross2camel } from '@linksmile-vue3-scaff/kit';

// Use Vuex
import { createStore } from 'vuex';

// Use Vuex Fast
import fast from 'vuex-fast';

// Export for Useage
export default (app, { util, store }, registry) => {
  /**
   * 特别说明：
   * vue-route 大部分情况仅需对 createRouter 之后的对象做拓展，故目前只考虑在 extension 回到函数中传入此实例
   * vuex 需要在其实例化之前之前做拓展，故目前只考虑在 extension 回调函数中传入 vuex 配置
   * ========== ========== ==========
   */

  // Get Modules from Store
  let modules = foreach(store, (hook, key) => {
    // Export by Renovate
    return {
      key: cross2camel(key),
      value: hook(util, registry.api),
    };
  });

  // Return as Extensor
  return extensor(
    // Before Init
    config => {
      // Extension Configure
      const configure = hand(config, modules, {});

      // Fast Modules
      modules = fast(modules);

      // $store
      return createStore({
        // Global
        ...registry.store(util),
        // Modules
        modules,
        // Configure
        ...configure,
      });
    },

    // After Init
    omni => {
      // Hack Api Update
      omni.update = (namespace, data = {}) => omni.dispatch(`${namespace}/STATE_UPDATE`, data);

      // For Use
      app.use(omni);
    },

    // API for Get
    modules,
  );
};
